#include <xc.h>
//이 Header 삽입문은 맨 처음 삽입해야 하는 삽입문으로 프로젝트에서 선택된
//PIC 마이크로컨트롤러의 헤더 파일로 연결되는 Header 삽입문이다.
//일반적으로는 #inlcude <pic16f1939.h> 로 삽입될 수도 있지만,
//PIC MCU가 변경시에는 모든 파일의 이름을 변경해야 하는 단점이 있다.

__CONFIG(FOSC_INTOSC & WDTE_OFF & PWRTE_ON & MCLRE_OFF & BOREN_OFF);
//위 설정은 내장된 오실레이터 16MHz를 사용하고, 와치독 타이머는 끄고,
//파워-업 타이머는 켜고, MCLR핀은 외부 리셋 기능 끄고,
//BOR 리셋 기능도 꺼진 상태로 설정된다.

//PICDEM 2 PLUS 보드를 사용시
#define PICDEM2PLUS
//#define INSEM_8bitPIC

//PIC MCU에서 사용하는 시스템 클럭 계산
#define Fosc 16000000 			//내부 16MHz 클럭 설정
#define Tosc 1/Fosc			//내부 16MHz 클럭 주기
#define Fcy Fosc / 4 			// Fcy = Fosc/4
#define Tcy 1 / Fcy 			// 1/4MHz = 250ns

// 캐릭터형 LCD, LED에 대한 I/O 포트 설정 선언문이다.
#ifdef PICDEM2PLUS
    //캐릭터형 LCD 포트 설정
    //LCD 컨트롤 핀 설정
    #define LCD_PWR	LATDbits.LATD7 // LCD ON/OFF 라인
    // 이 LCD_PWR 핀은 PICDEM 2 PLUS에만 있는 핀임

    #define LCD_EN 	LATDbits.LATD6 // LCD 컨트롤 활성화
    #define LCD_RW 	LATDbits.LATD5 // LCD 데이터 읽기/쓰기
    #define LCD_RS 	LATDbits.LATD4 // LCD 모드 선택

    //LCD 컨트롤 핀 설정
    #define LCD_DAT0    LATDbits.LATD0 	// Data bit 0
    #define LCD_DAT1    LATDbits.LATD1 	// Data bit 1
    #define LCD_DAT2    LATDbits.LATD2 	// Data bit 2
    #define LCD_DAT3    LATDbits.LATD3 	// Data bit 3
    #define LCD_DATA	LATD            // LCD 4비트 데이터버스

#endif
#ifdef INSEM_8bitPIC

    //캐릭터형 LCD 포트 설정
    //LCD 컨트롤 핀 설정
    #define LCD_EN	LATEbits.LATE2 // LCD 컨트롤 활성화
    #define LCD_RW	LATEbits.LATE1 // LCD 데이터 읽기/쓰기
    #define LCD_RS	LATEbits.LATE0 // LCD 모드 선택

    //LCD 컨트롤 핀 설정
    #define LCD_DATA	LATD			// LCD 8비트 데이터버스

#endif

//IO포트 초기화 함수
#ifdef PICDEM2PLUS
   void initial_port(void)
   {
	//PORTB 사용 LED 출력 포트 초기화
	TRISB = 0b11111110; //PORTB RB0핀을 출력으로 설정
	// 1 = 입력, 0 = 출력
	ANSELB = 0x00; 	//포트B의 ANxx 핀들 모두 디지털로 설정

	//캐릭터형 LCD을 위한 포트D 초기화
	TRISD = 0;		//포트 D의 RD0~RD7핀까지 모두 출력 설정
	ANSELD = 0;	//포트D의 ANxx 핀들 모두 디지털로 설정
   }

#endif

#ifdef INSEM_8bitPIC
   void initial_port(void)
   {
	//캐릭터형 LCD을 위한 포트D 초기화
	TRISD = 0;		//포트 D의 RD0~RD7핀까지 모두 출력 설정
	// 1 = 입력, 0 = 출력
	ANSELD = 0x00; 	//포트D의 ANxx 핀들 모두 디지털로 설정

	TRISEbits.TRISE0 = 0;	//RE0핀 LCD_RS 출력 설정
	TRISEbits.TRISE1 = 0;	//RE1핀 LCD_RW 출력 설정
	TRISEbits.TRISE2 = 0;	//RE2핀 LCD_EN 출력 설정
	ANSELE = 0x00; 	//포트E의 ANxx 핀들 모두 디지털로 설정

   }

#endif

//캐릭터형 LCD 컨트롤용 시간 delay 함수 소스
//마이크로초 딜레이 함수 define 문 정의
#define  DelayUs(x)  {  unsigned char _dcnt; \
                        _dcnt = 1; \
                        while(--_dcnt != 0) \
                        continue; }

//밀리초 딜레이 함수
void DelayMs(unsigned char cnt)
{
  unsigned char  i;
  do {
    i = 4;
    do {
      DelayUs(250);
    } while(--i);
  } while(--cnt);
}

//캐릭터 LCD의 명령, 데이터 모드 선언
#define LCD_COMMAND_MODE     1     // LCD Command
#define LCD_DATA_MODE        0     // LCD Data

#ifdef PICDEM2PLUS
//Microchip PICDEM 2 PLUS 보드 사용시에만 아래 LCD_ON(), LCD_OFF()함수 적용
//캐릭터형 LCD ON, OFF 함수 정의
#define LCD_ON()    LCD_PWR = 1     // LCD on
#define LCD_OFF()   LCD_PWR = 0     // LCD off

//캐릭터형 LCD 4비트 데이터 쓰기 함수
void write_4bit_data(unsigned char Lcd_command, unsigned char Lcd_data)
{
    unsigned char buffer;
	LCD_EN = 1;					//LCD On
	LCD_RW = 0;				//LCD 쓰기 모드
	if(Lcd_command)	LCD_RS = 0;		//LCD 명령 전송 모드
	else            LCD_RS = 1;		//LCD 데이터 전송 모드

	DelayMs(100);
        buffer = LATD;
        buffer &= 0xF0;
	LCD_DATA = buffer | (Lcd_data & 0x0F);		//LCD 명령 또는 데이터 전송
	DelayMs(100);
	LCD_EN = 0;					//LCD Off
}

void write_byte_data(unsigned char Lcd_command, unsigned char Lcd_data)
{
    write_4bit_data(Lcd_command, Lcd_data >> 4);  // Output the high nibble to the LCD
    write_4bit_data(Lcd_command, Lcd_data);       // Now send the low nibble
}

void LCD_Put_Data(unsigned char Lcd_data)
{
    write_byte_data(LCD_DATA_MODE,Lcd_data); // LCD에 문자를 출력을 보냄
    DelayMs(100); // 명령이 완료될 때까지 기다림
}

void LCD_Write_String(const char *Str)
{
    unsigned char i = 0; // 문자 인덱스 버퍼
    while (Str[i]) // 문자열이 완료되지않는동안 반복
    LCD_Put_Data(Str[i++]); // 최근 문자를 표시
}

void LCD_Clear(void)
{
    write_4bit_data(LCD_COMMAND_MODE ,0x00); // 초기화 디스플레이 명령을보냄
    write_4bit_data(LCD_COMMAND_MODE ,0x01); // 초기화 디스플레이 명령을보냄
    DelayMs(100); // 명령이 완료될 때까지 기다림
}

void LCD_Home(void)
{
    write_4bit_data(LCD_COMMAND_MODE ,0x00); // 초기화 디스플레이 명령을보냄
    write_4bit_data(LCD_COMMAND_MODE ,0x02); // 초기화 디스플레이 명령을보냄
    DelayMs(100); // 명령이 완료될 때까지 기다림
}


void LCD_Goto(unsigned Position, unsigned char Lcd_line)
{
    if ((Lcd_line > 1) || (Position > 15)) // 올바르지못한 선또는 열일경우
        return; // 그냥 아무것도하지 않음
    write_byte_data(LCD_COMMAND_MODE ,(Lcd_line == 1) ? (0xC0 | Position) : (0x80 | Position));
    // LCD_Goto 명령
    DelayMs(100); // 명령이 완료될 때까지 기다립니다.
}

//캐릭터형 LCD 초기화 함수
void initial_CLCD(void)
{
    LCD_OFF();
    LCD_ON(); // LCD를켜 Display에 표시
    LCD_RS = 0; // 명령 모드에서 LCD를 설정
    LCD_RW = 0; // LCD는 버스에서 데이터를 수신 => 쓰기 모드
    LCD_EN = 0; // LCD 사용 데이터 수신 (LCD는 Bus에서 무엇이든 읽어옴)
    LCD_EN = 1; // 요청시 LCD는 데이터를 설정합니다

    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x02);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x02);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x08);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x00);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x0C);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x00);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x01);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x00);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x02);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x04);
    DelayMs(100);
    write_4bit_data(LCD_COMMAND_MODE ,0x0E);
    DelayMs(100);

    LCD_Clear();
    LCD_ON(); // LCD를켜 Display에 표시
}
#endif

#ifdef INSEM_8bitPIC
#define LCD_STROBE()  ((LCD_EN = 1),(LCD_EN=0))
//write a byte to the LCD in 4 bit mode
void write_byte_data(unsigned char Lcd_data)
{
  DelayMs(100);
  LCD_DATA = Lcd_data;
  LCD_STROBE();
}

//Clear and home the LCD
void LCD_Clear(void)
{
  LCD_RS = 0;
  write_byte_data(0x1);
  DelayMs(100);
}

//write a string of chars to the LCD
void LCD_Write_String(const char *Str)
{
  LCD_RS = 1;        //write characters
  while(*Str)
    write_byte_data(*Str++);
}

//write one character to the LCD
void LCD_Put_Data(unsigned char Lcd_data)
{
  LCD_RS = 1;        //write characters
  write_byte_data( Lcd_data );
}


void LCD_Goto(unsigned Position, unsigned char Lcd_line)
{
    LCD_RS = 0;
    if ((Lcd_line > 1) || (Position > 15)) // 올바르지못한 선또는 열일경우
        return; // 그냥 아무것도하지 않음
    write_byte_data((Lcd_line == 1) ? (0xC0 | Position) : (0x80 | Position));
    // LCD_Goto 명령
    DelayMs(100); // 명령이 완료될 때까지 기다립니다.
}

//initialise the LCD - put into 8 bit mode
void initial_CLCD(void)
{
  LCD_RS = 0;
  LCD_EN = 0;
  LCD_RW = 0;

  DelayMs(100);       //wait 30mSec after power applied,
  LCD_DATA = 0b00111100;
  LCD_STROBE();
  DelayMs(100);
  LCD_DATA = 0b00001100;    //1100: 커서 없음, 1110 : 아래 바 커서, 1111: 커서 점멸
  LCD_STROBE();
  DelayMs(100);
  LCD_DATA = 0b00000001;
  LCD_STROBE();
  DelayMs(100);
  LCD_DATA = 0b00000110;
  LCD_STROBE();

  LCD_Clear();      //Clear screen
}
#endif

//타이머1 및 타이머1 인터럽트 초기화  함수 : 0.1초 인터럽트 설정
void initial_timer1_interrupt(void)
{
	TMR1H = 0; //타이머1 값 설정
	TMR1L = 0;
	TMR1IE = 0; //타이머1 인터럽트 설정 : 인터럽트 기능 Disable
	TMR1IF = 0; //타이머1의 인터럽트 플래그 초기화 -> 0
	T1CON = 0b00110000; //타이머1 프리스케일러 1:8
	TMR1L=(65536-50000)%256; //0.1초 카운트값 초기화
	TMR1H=(65536-50000)/256;
	TMR1IE = 1; //타이머1 인터럽트 설정 :인터럽트 기능 Enable 설정
	PEIE = 1; 	//타이머1의 주변장치 인터럽트 비트 : ON
	GIE =1; 	//전체 인터럽트 비트 : ON
	TMR1ON = 1; 	//타이머1 ON
}

//전역 변수 선언
unsigned char tmr1_count;  // 1초 인터럽트 회수 카운팅하는 전역 변수
unsigned char timer_sec;   // 전자 시계의 초를 카운팅하는 전역 변수
unsigned char timer_min;  // 전자 시계의 분을 카운팅하는 전역 변수
unsigned char timer_hour; // 전자 시계의 시간을 카운팅하는 전역 변수

//인터럽트 서비스 루틴 함수 및 1초 타이머 인터럽트 구동
void interrupt isr(void)
{
    if(TMR1IF == 1)
    {
        TMR1IF=0; //타이머1 인터럽트 플래그 다시 0을 만들어서 시간 재기
        TMR1L=(65536-50000)%256; //0.1초 카운트값 초기화
        TMR1H=(65536-50000)/256;
        tmr1_count++;
        if(tmr1_count==10)   //변수 값이 10 즉, 0.1초씩 10번이므로 1초가 됨
        {
            tmr1_count = 0; // 다시 10번 카운트하기 위해 리셋
	    timer_sec++;	  // 00 초에서 59 초까지 카운팅
        }
    }
}

//전자 시계를 동작하기 위한 시간, 분, 초 계산을 위한 함수 코딩
void    calculate_timer(void)
{
    if(timer_sec == 60)     //초가 60초가 되면 if문실행 if(timer_sec > 59)와 같음
    {
        timer_sec = 0;      //초 카운팅 변수를 다시 0초로 바꿈
        timer_min++;        //분 카운팅 변수를 +1 증가시킴
        if(timer_min == 60) //분이 60분이 되면 if문 실행 if(timer_min > 59)와 같음
        {
            timer_min = 0;  //분 카운팅 변수를 다시 0분으로 바꿈
            timer_hour++;   //시간 카운팅 변수를 +1 증가시킴
            if(timer_hour == 24)    //24시간이 되면 if문 실행 if(timer_hour > 23)와 같음
            {
                timer_hour = 0;     //시간 카운팅 변수를 다시 0시로 바꿈
            }
        }
    }
}

// 각 시, 분, 초를 10진수로 변환하여 LCD로 출력 함수 소스 코딩
void display_Lcd_dec_value(unsigned char time_value)
{
    unsigned char digit_10;     //십(10) 자리 표시 변수
    unsigned char digit_1;      //일(1) 자리 표시 변수

    digit_10    = (time_value/10)+'0';   //십의 자리 계산 + 0x30
    digit_1     = (time_value%10)+'0';   //일의 자리 계산 + 0x30

     LCD_Put_Data(digit_10);        //십의 자리 ASCII 값 LCD로 출력
     LCD_Put_Data(digit_1);         //일의 자리 ASCII 값 LCD로 출력
}

const char Lcd_timer_title[] = {"Elec. Watch"};

int main(int argc, char** argv)
{
    //오실레이터 초기화
    OSCCON = 0b01111000;	//16MHZ 내부 클럭 설정

    initial_port();             //LED를 위한 포트 초기화 함수 실행

    initial_timer1_interrupt(); //타이머1 인터럽트 동작 초기화 함수 실행

    //전역 변수
    tmr1_count=0;   // 1초 인터럽트 회수 카운팅하는 전역 변수 초기화
    timer_sec=0;    // 전자 시계의 초를 카운팅하는 전역 변수 초기화
    timer_min=0;    // 전자 시계의 분을 카운팅하는 전역 변수 초기화
    timer_hour=0;   // 전자 시계의 시간을 카운팅하는 전역 변수 초기화

    initial_CLCD();

    LCD_Write_String(Lcd_timer_title);
    //LCD 첫 라인에 "Elec. Watch" 표시

    LCD_Goto(0, 1);                 //0: 1번째 문자, 1: 두번째 라인 위치로 이동

    while(1)
    {
        calculate_timer();  //시, 분, 초 계산 함수 실행
        
        LCD_Goto(0, 1);        //0: 1번째 문자, 1: 두번째 라인 위치로 이동

        display_Lcd_dec_value(timer_hour);  //시간 표시
        LCD_Put_Data(':');
        display_Lcd_dec_value(timer_min);   //분 표시
        LCD_Put_Data(':');
        display_Lcd_dec_value(timer_sec);   //초 표시
         LCD_Put_Data(' ');
         
        if(timer_hour>11)           //00~11시 : AM 표시, 12~23시 : PM 표시
        {
            LCD_Put_Data('P');
            LCD_Put_Data('M');
        }
        else
        {
            LCD_Put_Data('A');
            LCD_Put_Data('M');
        }
    }
}



